;========================================================================
;                                                                       =
;                          Physics Solver v2.4                          =
;                                                                       =
; Written By:   Morgan Davies (morgazum85@gmail.com)                    =
;               Matt Bennett  (Mbcalcman@Yahoo.com)                     =
;                                                                       =
; Bio:  This program is relatively easy to follow, well except for the  =
;       math routines. Which were optimized and thus are really hard to =
;       follow. If you have any questions about this code, or need help =
;       with TI-86 assembly math, Morgan would be more than happy to    =
;       help you at the E-mail address above. If you borrow please give =
;       credit. Questions and comments are welcome at both E-mail       =
;       addresses listed above.                                         =
;                                                                       =
;========================================================================
;                                                 =                     =
;               Physics Solver v2.4               =   Equations List    =
;                      Forces                     =                     =
;                                                 =                     =
;   Size:   1045 Bytes                            =   1)  F=M*A         =
;                                                 =   2)  W=M*G         =
;   Keys:   Up      =   View Equations            =   3)  =Ff/Fn       =
;           1-4     =   Select Equation           =   4)  A=Fnet/M      =
;           F1-F3   =   Select Variable           =                     =
;           Exit    =   Exits Program             =                     =
;           Alpha   =   Shuts Off Calculator      =    = mu (greek)    =
;                                                 =                     =
;========================================================================


#include "ti86asm.inc"
#include "Asm86.h"
#include "Rom86.h"

.plugin lite86

_ioprompt   =$c324
_asap_ind   =$d623
_getcode    =$4068

.org _asm_exec_ram 

    nop                             ;no operation so...
    jp Start                        ;relative jump to Start
    .dw 0                           ;type of input
    .dw STR1                        ;print string

STR1:   .db "Physics Solver v2.4",0

Start:
    call _flushallmenus             ;clears all key codes
    call _runindicoff               ;turn off run indicator
    call Text1                      ;goes to Text1
    ld hl,STR2                      ;loads hl with string
    call _puts                      ;print string
    res 3,(iy+5)                    ;reverts text back to black on white
    ld hl,(256*1)+2                 ;loads coordinates into hl, row 2, col 3
    ld (_curRow),hl                 ;loads _curRow with hl
    ld hl,STR1                      ;loads hl with string
    call _puts                      ;print string
    ld hl,(256*2)+4                 ;loads coordinates into hl
    ld (_curRow),hl                 ;loads _curRow with hl
    ld hl,STR3                      ;loads hl with string
    call Text3                      ;goes to Text3
    call _getkey                    ;waits for a key to be pressed

Equations:
    call Text1                      ;goes to Text1
    ld hl,STR4                      ;loads hl with string
    call _puts                      ;print string
    res 3,(iy+5)                    ;reverts text back to black on white
    ld hl,(256*4)+2                 ;loads coordinates into hl
    ld (_curRow),hl                 ;loads _curRow with hl
    ld hl,STR5                      ;loads hl with string
    call Text3                      ;goes to Text3
    ld hl,$ff70                     ;loads coordinates into hl
    ld (hl),%10000000               ;what to display @ hl
    inc hl                          ;increases hl by 1
    ld (hl),%00000000               ;what to display @ hl
    inc hl                          ;increases hl by 1
    ld (hl),%00000001               ;what to display @ hl

LoopKey:
    call _getcode                   ;stores key pressed into a
GetKey:
    cp K_EXIT                       ;compares a with Exit key
    jp z,Exit                       ;if Exit jump to Exit
    cp K_1                          ;compares a with 1 key
    jr z,EQU1                       ;if 1 relative jump to EQU1
    cp K_2                          ;compares a with 2 key
    jr z,EQU2                       ;if 2 relative jump to EQU2
    cp K_3                          ;compares a with 3 key
    jr z,EQU3                       ;if 3 relative jump to EQU3
    cp K_4                          ;compares a with 4 key
    jp z,EQU4                       ;if 4 jump to EQU4
    cp K_UP                         ;compares a with Up key
    jr z,Equations                  ;if Up relative jump to Equations
    call Text7                      ;goes to Text7
    ld a,c                          ;loads a with c
    cp 1                            ;compares a with 1 key
    jr z,Equation1GetKey            ;if 1 relative jump to Equation1GetKey
    cp 2                            ;compares a with 2 key
    jr z,Equation2GetKey            ;if 2 relative jump to Equation2GetKey
    cp 3                            ;compares a with 3 key
    jr z,Equation3GetKey            ;if 3 relative jump to Equation3GetKey
    cp 4                            ;compares a with 4 key
    jp z,Equation4GetKey            ;if 4 jump to Equation4GetKey
    jr LoopKey                      ;relative jump to LoopKey

EQU1:
    call Text1                      ;goes to Text1
    ld hl,STR6                      ;loads hl with string
    call Text2                      ;goes to Text2
    ld hl,STR12                     ;loads hl with string
    call Text3                      ;goes to Text3
    ld hl,$ff73                     ;loads coordinates into hl
    ld (hl),%00000001               ;what to display @ hl

Equation1GetKey:
    call _getcode                   ;stores key pressed into a
    cp K_F1                         ;compares a with F1 key
    jp z,VAR1                       ;if F1 jump to VAR1
    cp K_F2                         ;compares a with F2 key
    jp z,VAR2                       ;if F2 jump to VAR2
    cp K_F3                         ;compares a with F3 key
    jp z,VAR3                       ;if F3 jump to VAR3
    call Text7                      ;goes to Text7
    ld c,1                          ;loads c with 1
    jr GetKey                       ;relative jump to GetKey

EQU2:
    call Text1                      ;goes to Text1
    ld hl,STR7                      ;loads hl with string
    call Text2                      ;goes to Text2
    ld hl,STR13                     ;loads hl with string
    call Text3                      ;goes to Text3
    ld hl,$ff74                     ;loads coordinates into hl
    ld (hl),%00000001               ;what to display @ hl

Equation2GetKey:
    call _getcode                   ;stores key pressed into a
    cp K_F1                         ;compares a with F1 key
    jp z,VAR4                       ;if F1 jump to VAR4
    cp K_F2                         ;compares a with F2 key
    jp z,VAR5                       ;if F2 jump to VAR5
    cp K_F3                         ;compares a with F3 key
    jp z,VAR6                       ;if F3 jump to VAR6
    call Text7                      ;goes to Text7
    ld c,2                          ;loads c with 2
    jp GetKey                       ;jumps to GetKey

EQU3:
    call Text1                      ;goes to Text1
    ld hl,STR8                      ;loads hl with string
    call Text2                      ;goes to Text2
    ld hl,STR14                     ;loads hl with string
    call Text3                      ;goes to Text3
    ld hl,$ff75                     ;loads coordinates into hl
    ld (hl),%00000001               ;what to display @ hl

Equation3GetKey:
    call _getcode                   ;stores key pressed into a
    cp K_F1                         ;compares a with F1 key
    jp z,VAR7                       ;if F1 jump to VAR7
    cp K_F2                         ;compares a with F2 key
    jp z,VAR8                       ;if F2 jump to VAR8
    cp K_F3                         ;compares a with F3 key
    jp z,VAR9                       ;if F3 jump to VAR9
    call Text7                      ;goes to Text7
    ld c,3                          ;loads c with 3
    jp GetKey                       ;jumps to GetKey

EQU4:
    call Text1                      ;goes to Text1
    ld hl,STR9                      ;loads hl with string
    call Text2                      ;goes to Text2
    ld hl,STR15                     ;loads hl with string
    call Text3                      ;goes to Text3
    ld hl,$ff76                     ;loads coordinates into hl
    ld (hl),%00000001               ;what to display @ hl

Equation4GetKey:
    call _getcode                   ;stores key pressed into a
    cp K_F1                         ;compares a with F1 key
    jp z,VAR10                      ;if F1 jump to VAR10
    cp K_F2                         ;compares a with F2 key
    jp z,VAR11                      ;if F2 jump to VAR11
    cp K_F3                         ;compares a with F3 key
    jp z,VAR12                      ;if F3 jump to VAR12
    call Text7                      ;goes to Text7
    ld c,4                          ;loads c with 4
    jp GetKey                       ;jumps to GetKey

VAR1:
    call SubVar9                    ;goes to SubVar9
    call SubVar1                    ;goes to SubVar1
    ld hl,E3                        ;print string
    call SubVar2                    ;goes to SubVar2
    ld hl,E1                        ;print string
    jp Text6                        ;jumps to Text6

VAR2:
    call SubVar10                   ;goes to SubVar10
    call SubVar4                    ;goes to SubVar4
    jp SubVar8                      ;jumps to SubVar8

VAR3:
    call SubVar9                    ;goes to SubVar9
    call SubVar4                    ;goes to SubVar4
    ld hl,E3                        ;print string
    jp Text6                        ;jumps to Text6

VAR4:
    call SubVar9                    ;goes to SubVar9
    call SubVar1                    ;goes to SubVar1
    ld hl,E5                        ;print string
    call SubVar2                    ;goes to SubVar2
    ld hl,E4                        ;print string
    jp Text6                        ;jumps to Text6

VAR5:
    call Text4                      ;goes to Text4
    ld hl,E5                        ;print string
    call SubVar5                    ;goes to SubVar5
    jp SubVar8                      ;jumps to SubVar8

VAR6:
    call SubVar9                    ;goes to SubVar9
    call SubVar5                    ;goes to SubVar5
    ld hl,E5                        ;print string
    jp Text6                        ;jumps to Text6

VAR7:
    call Text4                      ;goes to Text4
    ld hl,E8                        ;print string
    call SubVar6                    ;goes to SubVar6
    ld hl,E6                        ;print string
    jp Text6                        ;jumps to Text6

VAR8:
    call Text4                      ;goes to Text4
    ld hl,E6                        ;print string
    call SubVar1                    ;goes to SubVar1
    ld hl,E8                        ;print string
    call SubVar2                    ;goes to SubVar2
    ld hl,E7                        ;print string
    jp Text6                        ;jumps to Text6

VAR9:
    call Text4                      ;goes to Text4
    ld hl,E6                        ;print string
    call SubVar6                    ;goes to SubVar6
    ld hl,E8                        ;print string
    jp Text6                        ;jumps to Text6

VAR10:
    call SubVar9                    ;goes to SubVar9
    call SubVar7                    ;goes to SubVar7
    ld hl,E3                        ;print string
    jp Text6                        ;jumps to Text6

VAR11:
    call SubVar10                   ;goes to SubVar10
    call SubVar1                    ;goes to SubVar1
    ld hl,E2                        ;print string
    call SubVar2                    ;goes to SubVar2
    ld hl,E9                        ;print string
    jp Text6                        ;jumps to Text6

VAR12:
    call SubVar10                   ;goes to SubVar10
    call SubVar7                    ;goes to SubVar7
    jr SubVar8                      ;relative jump to SubVar8

SubVar1:
    call Input                      ;goes to Input
    rst 08h                         ;OP1 is moved to OP2
    ret

SubVar2:
    call Input                      ;goes to Input
    rst 28h                         ;OP1 * OP2 ==> OP1
    ret                             ;return

SubVar3:
    call Input                      ;goes to Input
    call _FPDIV                     ;OP1 / OP2  ==> OP1
    ret                             ;return

SubVar4:
    call SubVar1                    ;goes to SubVar1
    ld hl,E1                        ;print string
    call SubVar3                    ;goes to SubVar3
    ret                             ;return

SubVar5:
    call SubVar1                    ;goes to SubVar1
    ld hl,E4                        ;print string
    call SubVar3                    ;goes to SubVar3
    ret                             ;return

SubVar6:
    call SubVar1                    ;goes to SubVar1
    ld hl,E7                        ;print string
    call SubVar3                    ;goes to SubVar3
    ret                             ;return

SubVar7:
    call SubVar1                    ;goes to SubVar1
    ld hl,E9                        ;print string
    call SubVar3                    ;goes to SubVar3
    ret                             ;return

SubVar8:
    ld hl,E2                        ;print string
    jr Text6                        ;relative jump to Text 6

SubVar9:
    call Text4                      ;goes to Text4
    ld hl,E2                        ;print string
    ret                             ;return

SubVar10:
    call Text4                      ;goes to text4
    ld hl,E3                        ;oint string
    ret                             ;return

Text1:
    call _clrScrn                   ;clears screen
    set 3,(iy+5)                    ;inverts text to white text on black
    ld hl,(256*1)+0                 ;loads hl with coordinates
    ld (_curRow),hl                 ;loads _curRow with hl
    ret                             ;return

Text2:
    call _puts                      ;prints string
    res 3,(iy+5)                    ;reverts text back to black on white
    ld hl,(256*3)+2                 ;loads coordinates into hl
    ld (_curRow),hl                 ;loads _curRow with hl
    ld hl,STR10                     ;loads hl with string
    call _puts                      ;prints string
    ld hl,(256*1)+5                 ;loads coordinates into hl
    ld (_curRow),hl                 ;loads _curRow with hl
    ret                             ;return

Text3:
    call _puts                      ;prints string
    call DrawScreen                 ;goes to DrawScreen
    ret                             ;return

Text4:
    call _clrScrn                   ;clear screen
    call _homeup                    ;move to upper left
    ret                             ;return

Text6:
    call _puts                      ;prints string
    call DispAnswer                 ;goes to DispAnswer
    call _getkey                    ;waits for a key to be pressed
    jp Equations                    ;jumps to Equations

Text7:
    cp K_ALPHA                      ;compares a with Alpha key
    call z,CalcOff                  ;if Alpha jump to calcOff
    ret                             ;return

Input:
    ld de,_ioprompt                 ;$c324 - string storage for display when prompte
    call _mov10b                    ;$427b - move 10 bytes from hl to de
    ld a,$0d                        ;type of input
                                    ;$0d - # value
                                    ;$0c - string
    ld (_asap_ind),a                ;$d623 - tell ti86 what kind to be inputted
    jp _exec_pg3                    ;$5714 - ti86 os input
                                    ;the number inputted is always stored to OP1
    ret                             ;return

DispAnswer:
    ld a,48                         ;loads a with 48
    call _formReal                  ;forms answer to real #
    ld hl,_OP3                      ;loads answer into hl
    call _puts                      ;prints answer
    call _runindicoff               ;turn off run indicator
    ret                             ;return

CalcOff:
    res  3,(iy+8)                   ;lock the ADP
    set  4,(iy+8)                   ;expire ADP
    call $4101                      ;turns off calculator
    ret                             ;return

Exit:
    call _clrLCD                    ;clears the screen
    call _homeup                    ;moves cursor to upper left
    ret                             ;return that exits the program

STR2:   .db "       Forces      ",0   
STR3:   .db "By: Morgan Davies    "
        .db "    Matt Bennett",0
STR4:   .db "  Which Equation?  ",0
STR5:   .db "1)  F=M*A            "
        .db "2)  W=M*G            "
        .db "3)  =Ff/Fn          "
        .db "4)  A=Fnet/M",0
STR6:   .db "       F=M*A       ",0
STR7:   .db "       W=M*G       ",0
STR8:   .db "      =Ff/Fn      ",0
STR9:   .db "      A=Fnet/M     ",0
STR10:  .db "Solve For Which      "
        .db "   Variable?",0
STR11:  .db "EQU          1    2    3    4",0
STR12:  .db "F   M    A",0
STR13:  .db "W   M    G",0
STR14:  .db "   Ff   Fn",0
STR15:  .db "A  Fnet  M",0

E1: .db "F=?",0
E2: .db "M=?",0
E3: .db "A=?",0
E4: .db "W=?",0
E5: .db "G=?",0
E6: .db "=?",0
E7: .db "Ff=?",0
E8: .db "Fn=?",0
E9: .db "Fnet=?",0

;=================================
;Verticle Line Routine           =
;by Matt Bennett                 =
;Vars:  b=length                 =
;       c=line (%10000000)       =
;       hl=location in video mem =
;=================================

DrawLineVertical:
    ld de,$10                       ;loads de with $10
LineLoop:
    ld a,(hl)                       ;loads a with contents of video mem @ hl
    or c                            ;turns on the pixels specified by c w/o affecting others
    ld (hl),a                       ;loads a back into video mem
    add hl,de                       ;moves to next line
    djnz LineLoop                   ;repeat until b=0
    ret                             ;return

DrawLineHorizontal:
    ld (hl),%11111111               ;loads video mem @ hl w/ line
    inc hl                          ;moves one to the right
    djnz DrawLineHorizontal         ;repeat until line is drawn
    ret                             ;return
    
DrawScreen:
    ld hl,ULCorner                  ;loads "ULCorner" sprite
    ld de,$0000                     ;loads coordinates into de
    call GridPutSprite              ;display sprite
    ld hl,URCorner                  ;loads "URCorner" sprite
    ld de,$000f                     ;loads coordinates into de
    call GridPutSprite              ;display sprite
    ld hl,$fc80                     ;loads coordinates into hl
    ld b,48                         ;# of times to loop and display c
    ld c,%10000000                  ;what to display @ c
    call DrawLineVertical           ;draws left side line
    ld hl,$fc8f                     ;looads coordinates into hl
    ld b,48                         ;# of times to loop and display c
    ld c,%00000001                  ;what to display @ c
    call DrawLineVertical           ;draws right side line
    ld hl,$ff70                     ;loads coordinates into hl
    ld b,16                         ;# of times to loop and display hl
    call DrawLineHorizontal         ;draws bottom line
    ld hl,STR11                     ;string to print
    ld bc,256*56+3                  ;column 4, row 57
    ld (_penCol),bc                 ;loads c into _penCol
                                    ;  and b into _penRow
    call _vputs                     ;print it and return
    ld de,$0703                     ;loads start coordinates into de
    ld b,4                          ;# of menu boxes to draw

DrawMenuBoxes:
    push de                         ;gets variable de out of way for GPS
    push bc                         ;gets variable bc out of way for GPS
    ld hl,Menu                      ;loads "Menu" sprite
    call GridPutSprite              ;displays sprite
    pop bc                          ;retrieve variable bc
    pop de                          ;retrieve variable de
    inc de                          ;moves coordinates one to the right
    djnz DrawMenuBoxes              ;repeat
    ld hl,Left                      ;loads "Left" sprite
    ld de,$0700                     ;loads coordinates to de
    call GridPutSprite              ;displays sprite
    ld hl,$fff1                     ;loads coordinates to hl
    ld (hl),%11111111               ;what to display @ hl
    ld hl,Right                     ;loads "Right" sprite
    ld de,$0702                     ;loads coordinates to de
    call GridPutSprite              ;displays sprite
    ret                             ;return
    
GridPutSprite:
    push hl                         ; push hl to stack
    pop ix                          ; ...and pop it into iy
    srl d                           ; de = (128 * y) + x
    rra                             ; carry flag -< msb of a
    and $80                         ; a is either $00 or %80
    or e                            ; add x offset (remember x <= 15
    ld e,a                          ; de = (128 * y) + (x * 8)
    ld hl,$fc00                     ; hl-> video memory
    add hl,de                       ; hl-> video memory + offset
    ld b,8                          ; initialize loop counter
    ld de,16                        ; screen is 16 bytes wide

GPS_Loop:
    ld a,(ix + 0)                   ; get byte from sprite
    ld c,(hl)                       ; load c w/contents of video mem                    |These two lines are
    or c                            ; turn on pixels in sprite w/o turning off others   |my own additions
    ld (hl),a                       ; put byte on screen
    inc ix                          ; move to next byte in sprite
    add hl,de                       ; move to next line on screen
    djnz GPS_Loop                   ; loop until sprite is drawn
    ret                             ; return

Menu:
    .db %00000001
    .db %00000001
    .db %00000001
    .db %00000001
    .db %00000001
    .db %00000001
    .db %10000010
    .db %01111100

ULCorner:
    .db %00000111
    .db %00011111
    .db %00111111
    .db %01111111
    .db %01111111
    .db %11111111
    .db %11111111
    .db %11111111

URCorner:
    .db %11100000
    .db %11111000
    .db %11111100
    .db %11111110
    .db %11111110
    .db %11111111
    .db %11111111
    .db %11111111

Left:
    .db %10000000
    .db %10000000
    .db %10000000
    .db %10000000
    .db %10000000
    .db %10000000
    .db %01000000
    .db %00111111

Right:
    .db %00000001
    .db %00000001
    .db %00010001
    .db %00111001
    .db %01111101
    .db %00000001
    .db %00000010
    .db %11111100

.end
END